# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from novaclient import base
from novaclient.v1_1 import keypairs


class SavannaKeypairManager(keypairs.KeypairManager):
    def get(self, keypair):
        """Get a keypair.

        :param image: The ID of the keypair to get.
        :rtype: :class:`Keypair`
        """
        return self._get("/os-keypairs/%s" % base.getid(keypair), "keypair")
